/** moztorch.js
 * Methods required by moztorch.xul
 */

var g_machine;
var g_msgObserver = Components.classes["@pdx.edu/moztorch/mtMsgObserver;1"].
    createInstance( Components.interfaces.mtIMsgObserver );
var g_useThreads = false;


function dump(s) {
  g_msgObserver.handleMsg(s, Components.interfaces.mtIMsgObserver.MSG_SEVERITY_LOG);
  return;
}

//// XUL Methods //////////////////////////////////////////////////

var g_testFile, g_trainFile;
var g_machine, g_machineIsTrained;
var g_msgObserver = Components.classes["@pdx.edu/moztorch/mtMsgObserver;1"].
    createInstance( Components.interfaces.mtIMsgObserver );
/** Options associated with currently selected algorithm. */
var g_options;


function onLoadHandler() {
  // Set the target of log messages to main document textarea
  g_msgObserver.wrappedJSObject.logWindow = document.getElementById("logger");
  updateAlgorithm();
  //  debugCall('onLoadHandler');
}

function clearLogger() {
  document.getElementById("logger").value = "";
}

function startTrainPhase() {
  g_machineIsTrained = true;
  g_machine.batchTrain(g_trainFile.path);
  updateXulStates();
}

function startTestPhase() {
  // XXX won't work on mac!
  g_machine.batchTest(g_testFile.path);
}

function updateAlgorithm() {
  //  debugCall('updateAlgorithm');

  var machine_name = document.getElementById("algorithm-list").selectedItem.value;
  //  debugVar(machine_name, 'machine_name');

  g_machine = Components.classes["@pdx.edu/moztorch/mtMachine" + machine_name + ";1"];
  if (!g_machine)
    throw "Machine not found: " + machine_name;
  g_machine = g_machine.createInstance( Components.interfaces.mtIMachine );
  g_machine.logger = g_msgObserver;
  g_machineIsTrained = false;

  g_options =  machineOptionsToArray(g_machine);

  //  debugVar(g_options, 'options');

  // TODO add editable interface to option values

  var treeView = {
    _machine : g_machine,
    _options : g_options,
    rowCount : g_options.length,

    getCellText : function(row,column){
      var ret;
      if (column == "namecol")
      	return this._options[row][0];
      else if (column == "valuecol")
	return  this._options[row][1];
      return this._options[row][2];
    },
    setTree: function(treebox){ this.treebox = treebox; },
    isContainer: function(row){ return false; },
    isSeparator: function(row){ return false; },
    isSorted: function(row){ return false; },
    getLevel: function(row){ return 0; },
    getImageSrc: function(row,col){ return null; },
    getRowProperties: function(row,props){},
    getCellProperties: function(row,col,props){},
    getColumnProperties: function(colid,col,props){}
  };

  document.getElementById('algorithm-options-tree').view = treeView;

  updateXulStates();
}

function editMachineOption() {
  //  debugCall('editMachineOption');

  var tree = document.getElementById('algorithm-options-tree');
  var idx = tree.view.selection.currentIndex;

  //  debugVar(tree.treeBoxObject, 'tree', 1);
  //  debugVar(g_options, 'g_options');

  if (idx < 0)
    return;

  window.openDialog("chrome://moztorch/content/editMachineOption.xul", "emo-window", "chrome",
		    g_machine, g_options[idx][0], g_options[idx], tree.treeBoxObject);

  // Update value from machine
  //  g_options[idx][1] = getMachineOptionValue(g_machine, g_options[idx][0]);

  // Redraw effected row
  //  tree.treeBoxObject.invalidateRow(idx);
}

function loadDataFile(phase) {
  var fp = Components.classes["@mozilla.org/filepicker;1"]
    .createInstance(Components.interfaces.nsIFilePicker);

  var label;
  if (phase == "test")
    label = "Testing";
  else
    label = "Training";
  fp.init(window, "Select " + label + " File", fp.modeOpen);

  var res = fp.show();
  if (res == fp.returnOK) {
    document.getElementById(phase + 'File-text').value = fp.file.path;
    if (phase == "test")
      g_testFile = fp.file;
    else
      g_trainFile = fp.file;

    updateXulStates();
  }
}

function updateXulStates() {
  var trainButton = document.getElementById('startTrain-button');
  var testButton = document.getElementById('startTest-button');

  trainButton.disabled = true;
  testButton.disabled = true;

  if (g_machine) {
    if (g_trainFile)
      trainButton.disabled = false;
    if (g_testFile && g_machineIsTrained)
      testButton.disabled = false;
  }
}




//// Old Stuff /////////////////////////////////////////////////////////////

var rootPath  = "C:\\cygwin\\home\\thomure\\moztorch\\data\\personal.bd\\";
var nFeatures = 27;
var nClassValues = 2;
var trainPath = rootPath + "train.dat";
var testPath  = rootPath + "test.dat";
var modelPath = rootPath + "model.dat";

function testOptions(machine, trainPath, nFeatures, nClassValues) {
  debugCall("testOption");

  var count = { value: '' };
  var names = { value: '' };

  var opts = machine.getOptionNameEnum();

  dump("--- Machine Options ---");
  while (opts.hasMoreElements()) {
    var name = opts.getNext();
    var help = machine.getOptionHelp(name);
    var value = getMachineOptionValue(machine, name);

    if (help)
      help = "  // " + help;
    dump("  " + stringify(value, name) + help);
  }

  debugReturn("testOptions");
}


/** Running thread (if any). */
var g_thread;
/** Running thread type (if any). One of RUN_MODE_* */
var g_thread_type;

var mtIMachine = Components.interfaces.mtIMachine;
var nsIThread = Components.interfaces.nsIThread;

function makeRunnableMachine(machine, path, mode) {
  g_thread_type = mode;
  var runnable = machine.getRunnable(g_thread_type, path);

  g_thread = Components.classes["@mozilla.org/thread;1"].createInstance( nsIThread );
  g_thread.init(runnable, 0, nsIThread.PRIORITY_NORMAL, nsIThread.SCOPE_GLOBAL, nsIThread.STATE_JOINABLE);

  // XXX how to detect when thread has completed?
}

function testBatchTrain(machine, trainPath, nFeatures, nClassValues) {
  debugCall("testBatchTrain");

  if (g_useThreads) {
    // TODO add (monitored) check that g_thread_type == RUN_MODE_NONE (i.e. thread not runnin)
    makeRunnableMachine(machine, trainPath, mtIMachine.RUN_MODE_TRAIN);
    //    g_thread.join();
  } else {
    machine.batchTrain(trainPath);
  }

  debugReturn("testBatchTrain");
}

function testBatchTest(machine, testPath, mo) {
  if (g_useThreads) {
    // TODO add (monitored) check that g_thread_type == RUN_MODE_NONE (i.e. thread not running)
    makeRunnableMachine(machine, testPath, mtIMachine.RUN_MODE_TEST);
    //    g_thread.join();
  } else {
    machine.batchTest(testPath, mo);
  }
}

function testClassify(machine, pow) {
  var test = Array();
  for (var i=0; i < 9; ++i)
    test[i] = 10 * pow + i;
  return machine.classify(test.length, test);
}

function testMakeFile(path) {
  var file = Components.classes["@mozilla.org/file/local;1"]
	.createInstance(Components.interfaces.nsILocalFile);
  file.initWithPath(path);
  file = file.QueryInterface(Components.interfaces.nsIFile);
  return file;
}

function testMakeDS() {
  var dataset = Components.classes["@pdx.edu/moztorch/mtDataSet;1"]
	.createInstance(Components.interfaces.mtIDataSet);

  // Get direct access to enumerator javascript object
  var jsdataset = dataset.wrappedJSObject;
  if (!jsdataset)
    throw "DataSet is not a javascript object.";

  var data = Array();
  for (i=0; i < 10; ++i) {
    data[i] = Array();
    for (j=0; j < 10; ++j)
      data[i][j] = 10 * i + j;

    // keep target class within range [0,9]
    // XXX should be some check within Torch library on class values
    data[i][9] = i;
  }

  jsdataset.initFromArray(data);
  return dataset;
}


function testWriteDS(dataset, file) {
  dataset.writeToFile(file);
}

function testReadDS(file) {
  var testds = Components.classes["@pdx.edu/moztorch/mtDataSet;1"]
	.createInstance(Components.interfaces.mtIDataSet);
  testds.readFromFile(file);

  dump("testds; numRows: " + testds.getNumRows() + ", numCols: " + testds.getNumCols());
  var nrows = testds.getNumRows();
  //  var ncols = testds.getNumCols();
  for (i=0; i < nrows; ++i) {
    var arr = { value: null };
    var ncols = { value: null };
    testds.getColumn(i, ncols, arr);
    dump(arr.value.join(" "));
  }

  return testds;
}
