/** mtlib.js
 * Library of general moztorch routines.
 */


/***** DEBUGGING ROUTINES *******************/

/** Prints variable contents to dump(). */
function debugVar(obj, name, depth) {
  dump("DEBUG: " + stringify(obj, name, depth));
}

// TODO automate printing of caller name and param list
function debugCall(s) {
  dump("CALL: " + s);
}
// TODO automate printing of caller name
function debugReturn(s) {
  dump("RETURN: " + s);
}

/** Search given array (sequentially) for given object. 
 * @param arr array to search
 * @param o object to search for
 * @return index of object if found, else -1.
 */
function inArray(arr, o) {
  for (var i=0; i < arr.length; ++i)
    if (arr[i] == o)
      return i;
  return -1;
}

var g_stringifyActiveObjects = new Array();

/** Serialize variable to string buffer.
 * @param obj variable to serialize
 * @param name optional variable description
 * @param depth allowable depth to traverse
 * @param level current depth
 * @return created string
 * @TODO fix cyclic reference check (depth=2 crashes for treeView object)
 */
function stringify(obj, name, depth, level) {
  var type = typeof(obj);
  var buff = '', indent = '';

  // Set unlimited depth if unspecified
  if (depth == undefined)
    depth = -1;

  if (!level)
    level = 0;
  for (i=0; i < level; ++i)
    indent += "   ";

  if (inArray(g_stringifyActiveObjects, obj) >= 0) {
    buff += " !Cyclical Reference!"; // (" + obj.valueOf() + ")";
    depth = 0;
  }

  g_stringifyActiveObjects.push(obj);

  if (obj == null) {
    type = '';
    buff += 'null';

  } else if (type == "object") {
    if (obj.constructor == Array) {
      type = 'array';
      var obj_copy = new Array();
      if (depth) {
	for (var i in obj)
	  obj_copy[i] = stringify(obj[i], null, depth-1, level+1);
	buff += "[\n" + obj_copy.join("\n") + "\n" + indent + "]";
      }

    } else {
      type = "" + obj; // + "/" + obj.constructor.toString().replace(/[\[\]]/g, '');
      var t = new Array();
      // Check depth and avoid recurssion into xpconnect objects
      if (depth) {
	// Check for xpconnect object, attempting to unwrap underlying JS object
	if (type.search(/xpconnect/) >= 0) {
	  if (obj.wrappedJSObject)
	    t.push( stringify(obj.wrappedJSObject, null, depth-1, level+1) );
	} else {
	  for (var prop in obj)
	    try {
	      t.push( stringify(obj[prop], prop, depth-1, level+1) );
	    } catch (e) {
	      t.push( prop + "=unknown(error = " + e + ")" ); 
	    }
	}

	if (t.length == 0)
	  buff += "{}";
	else
	  buff += "{\n" + t.join("\n") + "\n" + indent + "}";
      }
    }

  } else if (type == 'function') {
    type = 'func()';
    //    buff += '';
  } else if (obj != undefined)
    buff += "(" + obj + ")";

  g_stringifyActiveObjects.pop();

  // Abbreviate type
  switch (type) {
  case 'number':
    type = 'num'; break;
  case 'boolean':
    type = 'bool'; break;
  case 'string':
    type = 'str'; break;
  }

  var ret = type + buff;
  if (name)
    ret = name + "=" + ret;

  return indent + ret;
}




/***** MOZTORCH-SPECIFIC ROUTINES ***********/

/** Lookup value of given option regardless of type. 
 * @param machine mtIMachine object
 * @param name string name of option
 * @return value of given option
 */
function getMachineOptionValue(machine, name) {
  //  debugCall('getMachineOptionValue(machine, ' + name + ')');

  type = machine.getOptionType(name);

  if (type == machine.OPT_TYPE_REAL)
    return machine.getROption(name);
  else if (type == machine.OPT_TYPE_INT)
    return machine.getIOption(name);

  return machine.getBOption(name);
}

/** Build list of option information tuples (NAME, VALUE, DESCRIPTION).
 * @param machine mtMachine to query
 * @return created array
 */
function machineOptionsToArray(machine) {
  var names = machine.getOptionNameEnum();
  var options = new Array();
  while (names.hasMoreElements()) {
    var name = names.getNext();
    options.push(new Array(name,
			   getMachineOptionValue(machine, name),
			   machine.getOptionHelp(name)
			   )
		 );
  }
  return options;
}

